package metadata_inventory;

import metadata_inventory_repository.AbstractRecordSet;
import metadata_model_mechanism.AbstractMechanism;


public class Attribute extends AbstractMetadataInventory {
	
	private SchemaData schema;	
	private AbstractRecordSet recordSetOrigin = null;
	
	// foi necessrio pois no caso do CSV o columnLabel tem ao format [tabela.campo]
	private String internalName = "";
    
	private tp_column_type column_type;
	
	private tp_data_type data_type;	
	private String data_value = "";
		
	private int column_size = 0;
	private String defaultValue = null;
	

	
    public static enum tp_data_type {
    	STRING, INTEGER, DOUBLE, FLOAT, LONG, BOOLEAN, DATE, SURROGATE_KEY
    }
    
    public static enum tp_column_type {
    	DATA, VIRTUAL
    }
    
    

	public Attribute(String name, String businessName, String definition, tp_data_type datatype, tp_column_type columntype, String defaultValue, int columnSize) {
		
		super(name, businessName, definition);
		
		this.internalName = name;
		this.data_type = datatype;
		this.column_type = columntype;
		this.defaultValue = defaultValue;
		this.column_size = columnSize;
	}
	
	public Attribute(String name, String columnBusinessName, SchemaData schema, tp_data_type dataType) {
		
		this(name, columnBusinessName,"",dataType,tp_column_type.DATA,null,0);
		
		this.internalName = name;
		this.data_type = dataType;
		this.schema = schema;		
	}
	
	
	public Attribute(String name, SchemaData schema, tp_data_type dataType) {
		
		this(name, name,"",dataType,tp_column_type.DATA,null,0);
		
		this.internalName = name;
		this.data_type = dataType;
		this.schema = schema;		
	}
	

	public Attribute(String name, SchemaData schema, int dataType) {
		
		this(name, name,"",tp_data_type.STRING,tp_column_type.DATA,null,0);
		
		this.internalName = name;		
		this.setData_type(dataType);		
		this.schema = schema;		
	}
	

	public Attribute(String name, AbstractMechanism transfOrigin) {
		
		this(name, name,"",tp_data_type.STRING,tp_column_type.DATA,null,0);
		
		this.internalName = name;		
		this.recordSetOrigin = transfOrigin.getRecordSetOutput();
	}
	
	
	public Attribute(String name, AbstractRecordSet rsOrigin, tp_data_type dataType) {
		
		this(name, name,"",dataType,tp_column_type.DATA,null,0);
		
		this.internalName = name;		
		this.recordSetOrigin = rsOrigin;		
	}


	

	public tp_data_type getData_type() {
		return data_type;
	}

	
	public tp_column_type getColumn_type() {
		return column_type;
	}

	
	public void setData_type(tp_data_type dataType) {
	
		this.data_type = dataType;
	}
	
	public void setData_type(int metadatacolumnType) {
		
		if (metadatacolumnType == 1 || metadatacolumnType == 12 || metadatacolumnType == -1) {
			this.data_type = tp_data_type.STRING;
		}
								
		else if (metadatacolumnType == 4 || metadatacolumnType == 5 || metadatacolumnType == -5) {
			
			this.data_type = tp_data_type.INTEGER;
		}
									 
		else if (metadatacolumnType == 2 || metadatacolumnType == 3 || metadatacolumnType == 8) { 
			
			this.data_type = tp_data_type.DOUBLE;
			
		}
		
		else if (metadatacolumnType == 6) { 
			 
			this.data_type = tp_data_type.FLOAT;
		}

		else if (metadatacolumnType == 7) { 
			 
			this.data_type = tp_data_type.LONG;	
			
		}
		
		else if ((metadatacolumnType == 91) ||  (metadatacolumnType == 93)) { 
			
			this.data_type = tp_data_type.DATE;
			
		}
		
		//System.out.println(this.getName() + " " + this.getData_type());
		
		/*-7	BIT
		-6	TINYINT
		-5	BIGINT
		-4	LONGVARBINARY 
		-3	VARBINARY
		-2	BINARY
		-1	LONGVARCHAR
		0	NULL
		1	CHAR
		2	NUMERIC
		3	DECIMAL
		4	INTEGER
		5	SMALLINT
		6	FLOAT
		7	REAL
		8	DOUBLE
		12	VARCHAR
		91	DATE
		92	TIME
		93	TIMESTAMP
		1111 	OTHER*/
		
		
	}

	
	public Attribute(String name, String definition, tp_column_type columnType) {
		super(name, definition);
		column_type = columnType;
	}


	public SchemaData getSchema() {
		
		return schema;
	}
	
	
	public String getSchemaName() {
		
		String cSchema = "";
		
		if (this.schema != null) 
			cSchema = this.schema.getName();
		else
		{
		    if (this.recordSetOrigin != null) {
		    	
		    	//  melhor que d erro
		    	//if (this.recordSetOrigin.getSchemaData() != null)
		    	
		    	   cSchema = this.recordSetOrigin.getSchemaData().getName();		    	
		    }
			   
		}
			
				
		return cSchema;
	}


	public void setSchema(SchemaData schema) {
		this.schema = schema;
	}

	public AbstractRecordSet getRecordSetOrigin() {
		return recordSetOrigin;
	}


	public void setRecordSetOrigin(AbstractRecordSet recordSetOrigin) {
		this.recordSetOrigin = recordSetOrigin;
	}

	public String getDefaultValue() {
		return defaultValue;
	}

	public void setDefaultValue(String defaultValue) {
		this.defaultValue = defaultValue;
	}

	public int getColumn_size() {
		return column_size;
	}

	public void setColumn_size(int columnSize) {
		column_size = columnSize;
	}

	public String getData_value() {
		return data_value;
	}

	public void setData_value(String dataValue) {
		data_value = dataValue;
	}

	public String getInternalName() {
		return internalName;
	}

	public void setInternalName(String internalName) {
		this.internalName = internalName;
	}



	
	
}
