package metadata_inventory_repository;


import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import java.util.ArrayList;
import java.util.List;


import metadata_inventory.Attribute;
import metadata_inventory.Environment;
import metadata_inventory.SchemaData;
import metadata_inventory.Attribute.tp_column_type;
import metadata_inventory.Attribute.tp_data_type;
import metadata_inventory_repository.AbstractSQLInstruction;
import metadata_model_mechanism.ListOfMapper;
import metadata_model_mechanism.Mapper;

public abstract class AbstractRecordSet {
		
	private Environment environment = null;
	private ResultSet data_Cache = null;
	private PreparedStatement data_Cursor = null;
		
	private AbstractSQLInstruction sqlInstruction = null;
	
	private ResultSetMetaData schemaMetadata = null;
	
	private List<Attribute> columns = null;
			
	public abstract SchemaData getSchemaData();
	
	
	public ResultSet getRecordSet() {

		if (this.data_Cache == null)
			this.refreshDataCache();
		
		return this.data_Cache;		
	}
	
	
	
	
	public AbstractRecordSet(Environment environment) {
		super();
		this.environment = environment;
	}


	public String getColumnVal(Attribute col) {
		
		try {
			return this.getRecordSet().getString(col.getName()).toString();
			
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return "";		
	}

	

	
	
	
	
	public void refreshDataCache() {
		
		this.data_Cache = this.getEnvironment().getRepository().getDataSQL(this.sqlInstruction.getSelect());
		
		this.refreshMetadata();
				
		this.defineColumnsTypes(this.getMetadata());

						
	}
	
	
	
	
	public ResultSetMetaData getMetadata() {
		
		return this.schemaMetadata;
	}
	
	
	public void refreshMetadata() {
		
		try {
						
			this.schemaMetadata = this.data_Cache.getMetaData(); 
			
			
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			System.out.println("Erro ao tentar capturar metadata");
			//e.printStackTrace();
		}
		
	}
	
	
	
	public void refresDataCursor(ListOfMapper  mappers) {		
		
		this.setData_Cursor(this.getEnvironment().getRepository().getStatement(this.sqlInstruction.getInsert(mappers)));	
	}
	
	
	
	

	public PreparedStatement getDataCursor(ListOfMapper  mappers) {
		
		if (this.data_Cursor == null)
			this.refresDataCursor(mappers);
		
		return this.data_Cursor;		
	}
	
	
	public void prepareCursor(String outPutSchema,ListOfMapper mappers) {
	
		this.setSqlGenerator(outPutSchema);
		
		/* OBS: ISTO FOI UMA MARTELADA PARA GARANTIR QUE A ORDEM DOS CAMPOS NO INSERT
		 * SEJA A MESMA ORDEM DOS CAMPOS EM this.recordSetOutput.getColumns() 
		 * QUE  PREENCHIDA EM prepareOutputStructure, e assim garantir que o executeRow
		 * seja executado corretamente 
		 * mas pode-se tentar refatorar isto. */
		
		for (int x = 0; x < mappers.size(); x++) {
			
			// em 28/01
			
			// mario domingo de carnaval
			if (mappers.get(x).isColOutORColOutCalculated())
				continue;
			
			Attribute col = mappers.get(x).getColumn_out();
			
			if (col == null) {
				col = mappers.get(x).getColumn_in();
			}
						
			//if (col.getColumn_type() == tp_column_type.DATA) {
			this.sqlInstruction.getSqlElements().addField(col,"");
				
			System.out.println("INCLUI FILD DATA: " + col.getName().toString());
			
		}	
		
		
		for (int x = 0; x < mappers.size(); x++) {
			
			// em 28/01
			
			// mario domingo de carnaval
			if (!mappers.get(x).isColOutORColOutCalculated())
				continue;
			
			
			Attribute col = mappers.get(x).getColumn_out();			
			if (col == null) {
				col = mappers.get(x).getColumn_in();
			}
			
			//if (col.getColumn_type() == tp_column_type.CALCULETED) {
			this.sqlInstruction.getSqlElements().addField(col,"");
			System.out.println("INCLUI FILD CALC: " + col.getName().toString());				
			
			   
			//System.out.println("mapper: " + this.getMapper(x).getColumn_in().getName());
		}	
					
	}
	
	/*
	public void prepareCursor(String outPutSchema,List<String> fieldsOut) {
		
		this.setSqlGenerator(outPutSchema);				
		
		for (int x = 0; x < fieldsOut.size(); x++) {
			this.sqlGenerator.getSqlElements().addField(new Column(fieldsOut.get(x)));
			//System.out.println("mapper: " + this.getMapper(x).getColumn_in().getName());
		}											
		
	}
	*/

		

	public Environment getEnvironment() {
		return environment;
	}

	public void setEnvironment(Environment environment) {
		this.environment = environment;
	}


	public AbstractSQLInstruction getSqlGenerator() {
		return sqlInstruction;
	}


	public void setData_Cursor(PreparedStatement dataCursor) {
		data_Cursor = dataCursor;
	}


	public void setSqlGenerator(String table) {
		
		this.sqlInstruction = this.createSqlSyntax(table);
		
	}
		
	
	public AbstractSQLInstruction createSqlSyntax(String table) {
		// TODO Auto-generated method stub
		
		return this.getEnvironment().getRepository().createLogicalInstruction(table);
						
	}
	
	

	public void defineColumns(java.sql.ResultSetMetaData metadataModel, SchemaData schemadata, ListOfMapper mappers) {
			
		this.columns = new ArrayList<Attribute>(); 
		
		try {
			
			System.out.println("colunas::::::::::::: " + metadataModel.getColumnCount());
			
			for (int x = 1; x <= metadataModel.getColumnCount();x++) {
				
				/* deu bronca com CSV pois, quando usa arquivo CSV o label do campo
		     	vem sempre = [tabela.campo], assim precisamos tambm guardar o nome interno
		     	usando col.setInternalName */
				//String colName = metadataModel.getColumnLabel(x);
				
				//String colInternalName = metadataModel.getColumnLabel(x);
				
				String colName = metadataModel.getColumnLabel(x);	// deu bronca com CSV
				
				// pois no CSV o campo em no formato [campo]
				colName = colName.replaceAll("\\[", "");
				colName = colName.replaceAll("\\]", "");
				
				
				
				//String colName = metadataModel.getColumnName(x);	// deu bronca com CSV			
				
				//System.out.println("coluna:               " + metadataModel.getColumnName(x));
				
				
				/*os tratamentos abaixo foram necessarios pois, quando usa arquivo CSV o label do campo
			     	vem sempre = [tabela.campo]   				
				colName = colName.replaceAll("]","");				
				if (colName.contains("."))						
					colName = colName.substring(colName.indexOf(".") +1, colName.length());
					*/	

				// em 04/03, comentei o codigo comentado abaixo
				/*
				if (mappers.findMapperByColOut(colName) != null) 				
		    	if (mappers.findMapperByColOut(colName).getColumn_out().getColumn_type() == tp_column_type.CALCULETED) {
					System.out.println("No considerou Coluna " + colName + " Output pois  calculada definecolumns");
					continue;
				}
				*/
		    	
				
				// em 28/11
		    	if (mappers.findMapperByColOut(colName) == null) {
					System.out.println("No considerou Coluna " + colName + " Output NULL em definecolumns");
					continue;
				}
		    	
		    	if (mappers.findMapperByColOut(colName).getColumn_out().getColumn_type() == tp_column_type.VIRTUAL) {
					System.out.println("No considerou Coluna " + colName + " Output pois  calculada definecolumns");
					continue;
				}
				
				
				System.out.println("coluna criada::::::::::::: " + colName);
								
				// mudana mario
				Attribute col = new Attribute(colName,schemadata,metadataModel.getColumnType(x));
				
				col.setInternalName(colName);
				//mappers.findMapperByColOut(colName).getColumn_out().setInternalName(colInternalName);
				
				this.columns.add(col);
								
			}
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	
	public void defineColumnsTypes(java.sql.ResultSetMetaData metadataModel) {
		
				
		
		try {
			System.out.println("Colunas do METADATA:::::::" + metadataModel.getColumnCount());
		} catch (SQLException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		
		try {
			for (int x = 1; x <= metadataModel.getColumnCount();x++) {
				
				// mudana mario  OBS: METADATAMODEL SEMPRE COMEA COM INDEX = 1
				Attribute col = this.getColunm(metadataModel.getColumnName(x));  
				
				if (col != null)				
				   col.setData_type(metadataModel.getColumnType(x));
								
			}
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		
	}



	public List<Attribute> getColumns() {
		
		return columns;
	}


	public void setColumns(List<Attribute> columns) {
		this.columns = columns;
	}
	
	
	public void fillColumnsValue() {
		
		ResultSet data = this.getRecordSet();

		for (int i = 0; i < this.columns.size();i++) {
		
			Attribute col = this.columns.get(i);
			
			col.setData_value("");
			
						
			// se for campo calculado vai dar erro
			try{					
			    //System.out.println(data.findColumn(col.getName()));
			    
				col.setData_value(data.getString(col.getName()));
			    
			} catch (SQLException e) {
				System.out.println("Erro ao tentar consultar dataset input: " + e.toString());	
			}														
		}
	}
	
	
	
	
	public boolean firstRow(boolean fillcolumns) {		
		boolean l;
		
		try {
			l = this.getRecordSet().first();
			
			//System.out.println("Firsssssssssssssst");
			
			if ((fillcolumns) && (l))
			   this.fillColumnsValue();
			
			return l;
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;		
	}
	
	
	public boolean nextRow(boolean fillcolumns) {
		boolean l;
		
		try {
			l = this.getRecordSet().next();
			
			//System.out.println("NeXXXXXXXXXXXXXXXXXXXXxt");
			
			if ((fillcolumns) && (l))			
			  this.fillColumnsValue();
			
			return l; 
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
		
	}
	
	
	public boolean previostRow(boolean fillcolumns) {
		boolean l;
		
		try {
			l = this.getRecordSet().previous();
			
		//	System.out.println("previosssssssssssssssssss");
			
			if ((fillcolumns) && (l))
			   this.fillColumnsValue();
			
			return l;
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
		
	}
	
	public Attribute getColunm(String name) {
		
		for (int x =0;x < this.columns.size();x++) {
			
			if (this.columns.get(x).getName().equalsIgnoreCase(name))
				return this.columns.get(x);			
			
		}
				
		return null;				
	}
	
	
	
	
	public Attribute addColumn(String colName)  {
		 
		if (this.columns == null)
			this.columns = new ArrayList<Attribute>();
		
		Attribute col = new Attribute(colName,this.getSchemaData(),tp_data_type.STRING);
		this.columns.add(col);
		

		
		this.getSqlGenerator().getSqlElements().addField(col,colName,true,"");
		
		return col;
	}
	
	
	
	public void addColumnsFromSchema() {
		
		
		if (this.getSqlGenerator() == null) 
		    this.setSqlGenerator(this.getSchemaData().getName());
		
		
		ResultSet resultSet = this.getEnvironment().getRepository().getDataSQL(this.sqlInstruction.getSelectAll());
					
		try {
			java.sql.ResultSetMetaData mtModel = resultSet.getMetaData();
		    System.out.println("Colunas do METADATA:::::::" + mtModel.getColumnCount());
		    
		    for (int x = 1; x <= mtModel.getColumnCount();x++) {
				
				Attribute col = this.addColumn(mtModel.getColumnName(x));									
				col.setData_type(mtModel.getColumnType(x));					
			}
		    
		} catch (SQLException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	
		
	}
	
	
	
}
