package metadata_inventory_repository;


import java.util.ArrayList;
import java.util.List;

import metadata_inventory.Attribute;
import metadata_inventory.SchemaData;
import metadata_inventory.Attribute.tp_column_type;
import metadata_model_mechanism.ListOfMapper;
import metadata_model_mechanism.ListOfParams;


public class SQLElements {
	
	protected List<String> tables = new ArrayList<String>();	
	protected List<Field>  fields = new ArrayList<Field>();	
	protected List<String> joins = new ArrayList<String>();
	protected List<String> where = new ArrayList<String>();
	protected List<String> groupBy = new ArrayList<String>();
	protected List<String> having = new ArrayList<String>();
	protected List<String> orderBy = new ArrayList<String>();
		
	
	public SQLElements(String table) {
		super();		
		this.addTable(table);
	}
	
	
	
	public void addTables(List<SchemaData> lsttables) {
		
		this.tables.clear();
		for (int x = 0; x < lsttables.size() ;x++) {
			
			this.tables.add(lsttables.get(x).getName());			
		}
	}

	
	public void addField(Attribute column, String fieldAlias) {
			 		
		 this.addField(column,fieldAlias,true,"");
	}
	

	public void addField(Attribute column, String fieldAlias, boolean lUseSchemaFromColumn, String function) {
		
		 String tb = "";
				 
		if (lUseSchemaFromColumn && (column.getSchemaName() != null))
			 tb = column.getSchemaName();
		 else		 
		     tb = this.getTable(0);
		
		 this.fields.add(new Field(column.getName(),fieldAlias,tb,function));		
	}
	

	
	
	public void addFields(ListOfMapper mappers, boolean lUseSchemaFromColumn) {
		
		for (int x = 0; x < mappers.size(); x++) {
			
			if (mappers.get(x).getColumn_in() == null) {
				System.out.println("nao considerou no SQL coluna nula de indice: " + x);
				continue;
			}
										
			if (mappers.get(x).getColumn_in().getColumn_type() == tp_column_type.VIRTUAL) {				
				System.out.println("nao considerou no Select campo calculado AQUI: " + mappers.get(x).getColumn_in().getName());
				continue;
			}
			
		
			// em 28/01
			String cColAs = mappers.get(x).getColumn_in().getName();			
			if (mappers.get(x).getColumn_out() != null) {
				cColAs = mappers.get(x).getColumn_out().getName();
			}
			 
						
			this.addField(mappers.get(x).getColumn_in(),cColAs,lUseSchemaFromColumn,"");			
			//System.out.println("mapper: " + this.getMapper(x).getColumn_in().getName());
		}
		
		
		
		
		
		
	}
	

	
	
	
	public void addWhere(String where) {
		
		this.where.add(this.tables.get(0) + "." + where);
	}
	
	
	public void addWhereFilter(String fieldExpression, String operator, String value) {
		
		if (value.toUpperCase().equalsIgnoreCase("NULL")) {
			
			if (operator.equalsIgnoreCase("="))
				operator = "IS";
			else
				operator = "IS NOT";
		} 
		else
			value = " '" + value + "' ";
					
				 
		this.addWhere(fieldExpression + " " + operator + "  " + value);
				
	}
	
		
	public void addJoins(String join) {
		this.joins.add(join);
	}


	public void addGroupBy(String groupBy) {
		// this.groupBy.add(this.tables.get(0) + "." +groupBy);
		this.groupBy.add(groupBy);
	}
	

	public void setHaving(String having) {
		this.having.add(this.tables.get(0) + "." +having);
	}


	public void addOrderBy(String orderBy) {
		this.orderBy.add(this.tables.get(0) + "." +orderBy);
	}

	

	public List<Field> getFields() {
		return this.fields;
	}


	public Field findField(String tableName, String fieldName) {
		
		for (int x =0; x < this.getFields().size(); x++) {
			
			if (this.fields.get(x).getFieldName().equalsIgnoreCase(fieldName) && this.fields.get(x).getFieldTable().equalsIgnoreCase(tableName)) {	
				return this.fields.get(x);			
			}
			
		}
				
		return null;				
	}
	

	public String getTable(int index) {
		return tables.get(index);
	}
	
	public void addTable(String table) {
		tables.add(table);
	}
	



	
	public void addJoinRelation(int relation1_tableIndex, String relation1_field, int relation2_tableIndex, String relation2_field) {
		 
		this.addJoins("JOIN " + this.getTable(relation2_tableIndex) + " ON (" + this.tables.get(relation1_tableIndex) + "." + relation1_field + " = " + this.getTable(relation2_tableIndex) + "." + relation2_field + ")");
				
	}
	
	
	public void addJoinRelation(String relation1, String relation1_field, String relation2, String relation2_field) {
		 
		this.addJoins("JOIN " + relation2 + " ON (" + relation1 + "." + relation1_field + " = " + relation2 + "." + relation2_field + ")");
				
	}
	
	
	
	
	
	public void putFunction(String fieldExpression, String function) {
		
		Field field = this.findField(this.getTable(0),fieldExpression);
		
		if (field != null) {
			field.setFunction(function);

		}
		else {
			
			this.getFields().add(new Field(fieldExpression,fieldExpression,this.getTable(0),function));
			
			//this.addField(column, fieldAlias, lUseSchemaFromColumn, function)
		
		}		
		
	}
	
	
	
	public void putGroupBy(ListOfParams params) {
		
		for (int x = 0;x < this.getFields().size();x++) {
			
			int i;
			
			i = params.findItemByValue(0, this.getFields().get(x).getFieldName());
			
			if (i == -1) 				
				this.addGroupBy(this.getFields().get(x).getFieldTable() + "." + this.getFields().get(x).getFieldName());			
		}
		
	}
	



	

	

}
