package metadata_model_general;

import java.util.ArrayList;
import java.util.List;

import javax.xml.validation.Schema;


import metadata_inventory.Attribute;
import metadata_inventory.Environment;
import metadata_inventory.SchemaData;
import metadata_inventory_repository.AbstractRepositoryJDBC;
import metadata_model_general.*;
import metadata_model_mechanism.AbstractMechanism;
import metadata_model_mechanism.ListOfMapper;
import metadata_model_mechanism.Mapper;

import frametl.FramETL;

public class Job extends AbstractMetadata implements Performable {
	
//	protected ListOfMapper mappers = new ListOfMapper();
	
	Environment environment_Source = null;
	Environment environment_Target = null;

	private List<SchemaData> schemaData_source = new ArrayList<SchemaData>();
	private List<SchemaData> schemaData_target = new ArrayList<SchemaData>();

//	private SchemaData lastOutput = null;

	private List<AbstractMechanism> transforms = new ArrayList<AbstractMechanism>();

	public Job(String name, String definition) {
		super(name, definition);
		// TODO Auto-generated constructor stub
	}

	public AbstractMechanism addMechanism(AbstractMechanism mechanism) {
		this.transforms.add(mechanism);
		
		System.out.println("Transformation Added. "  + mechanism.getName());
		
		return mechanism;
	}

	public AbstractMechanism getTransform(int index) {
		return transforms.get(index);
	}
	
	public int getTransformsCount() {
		return transforms.size();
	}

	

	public void addSchema_source(SchemaData schema) {
		this.schemaData_source.add(schema);
	}

	public void addSchema_target(SchemaData schema) {
		this.schemaData_target.add(schema);
	}
	
	

	public SchemaData getSchema_source(int index) {
		return this.schemaData_source.get(index);
	}
	
	
	public Attribute getColumn_source(int indexSource, String columnName) {
				
		return this.schemaData_source.get(indexSource).findColumn(columnName);			
		
	
	
	}
	
	public Attribute getColumn_source(String columnName) {
		
		return this.schemaData_source.get(0).findColumn(columnName);			
		
	}

	
	public Environment getEnvironment_Source() {
		return environment_Source;
	}

	public void setEnvironment_Source(Environment environmentSource) {
		environment_Source = environmentSource;
	}

	public Environment getEnvironment_Target() {
		return environment_Target;
	}

	public void setEnvironment_Target(Environment environmentTarget) {
		environment_Target = environmentTarget;
	}

	
	@Override
	final public boolean perform() {
		// TODO Auto-generated method stub

		for (int x = 0; x < transforms.size(); x++) {

			System.out.println("");
			System.out
					.println("----------------------------------------------------------------------");
			System.out.println(">>> Performing Transformation: " +  transforms.get(x).getName());

			/*
			if (x > 0) 
			   transforms.get(x).identifyInputSchemas(transforms.get(x-1));
			else
			   transforms.get(x).identifyInputSchemas(null);
			
			transforms.get(x).identifyInputColumns();

			transforms.get(x).onOpenDataSet();

			transforms.get(x).prepareSchemaOutput();
			
			*/
			
			if (x > 0) 
				transforms.get(x).execute(transforms.get(x-1));
			else
				transforms.get(x).execute(null);
						
		}

		return false;
	}

	
	
	
	public SchemaData getSchemaDataSource(int index) {
		return this.getEnvironment_Source().getSchemaData(index);
	}
	
	
	public SchemaData getSchemaDataTarget(int index) {
		return this.getEnvironment_Target().getSchemaData(index);
	}
	
	public SchemaData getSchema_target(int index) {
		return this.schemaData_target.get(index);
	}


	public SchemaData getSchemaFromEnvSource(String name) {

		return this.getEnvironment_Source().findSchemaData(name);

	}

	public SchemaData getSchemaFromEnvTarget(String name) {

		return this.getEnvironment_Target().findSchemaData(name);

	}

	public AbstractMechanism findTransformation(String name) {

		for (int x = 0; x < this.transforms.size(); x++) {

			if (this.transforms.get(x).getName().equalsIgnoreCase(name))
				return this.transforms.get(x);

		}

		return null;

	}


}
