package metadata_model_mechanism;

import frametl.FramETL;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;

import metadata_inventory.Attribute;
import metadata_inventory.Environment;
import metadata_inventory.SchemaData;
import metadata_inventory.Attribute.tp_column_type;
import metadata_inventory.Attribute.tp_data_type;
import metadata_inventory_repository.AbstractSQLInstruction;
import metadata_inventory_repository.RecordSetInput;
import metadata_inventory_repository.RecordSetOutput;
import metadata_inventory_repository.SQLElements;
import metadata_inventory_repository_common.RepositoryXML;
import metadata_model_general.*;
import metadata_model_mechanism.RowControl.tp_flow_sequence;
import metadata_model_mechanism.RowControl.tp_status;
import metadata_model_mechanisms_common.Conversion;

public abstract class AbstractMechanism extends AbstractMetadata implements MechanismEvents {
	
	// attributes
	protected ListOfMapper mappers = new ListOfMapper();		
	protected ListOfParams params = new ListOfParams();
			
	private RecordSetInput recordSetInput = null;	
	private RecordSetOutput recordSetOutput = null;
	
	private List<MechanismEvents> customHandle = new ArrayList<MechanismEvents>();
	
	private Job internalJob = null;
	
	
	// methods
	protected void addParam(Params param) {		
		this.params.add(param);		
	}
		
	public Mapper addMapper(Attribute column_in, Attribute column_out) {
		Mapper map = this.getMappers().findMapperByColOut(column_out.getName());
		
		if (map != null)
			return map;
		   
		this.getMappers().add(column_in, column_out);
		
		map = this.getMappers().get(this.getMappers().size()-1);
		
		return map;
	}
	
	public void addCustomHandler(MechanismEvents tansformationListener) {
		this.customHandle.add(tansformationListener);
	}

	
	
	
	
	
	
	private List<AbstractMechanism> transfInput = new ArrayList<AbstractMechanism>();	
	
	private PreparedStatement objOutput = null;
	
	public boolean l_loadAllInputAttributes = true;
	
	private Job jobOwner = null;	
		
	private String tag = "";
	
	private int nAcceptedRecords = 0; 
	
	
	// implementado na classe abstrata
	public abstract boolean identifyInputColumns();		

	



	 
	// metodos hooks ha ordem em que so chamados pelo frameETL
	
	public abstract void init();
	

	
	/*  Alternativa ao uso de transformaes baseadas em SQL para carrego em memoria
	public List<List<String>> tablevaluesIn = new ArrayList<List<String>>();
	public List<List<String>> tablevaluesOut = new ArrayList<List<String>>();
		*/	

	
	
			
	final public void identifyInputSchemas(AbstractMechanism last_transformation) {
		
		// se o usuario tiver definido os schemas explicitamente NO FAZ NADA
		if (this.getSchemasInput().size() == 0) {
					
			// se NO tiver definido as transformaes de ORIGEM ento usa o schema do ULTIMA TRANSFORMAO 
			if (this.getRecordSetInput().getRecordSetsOrigin().size() == 0) {
				
				if (last_transformation != null) {
				   this.getRecordSetInput().addSchema(last_transformation.getSchemaOutput());
				   this.transfInput.add(last_transformation);
				}
				
				
			}	
			else {
				
				// se tiver definido as transformaes do ORIGEM ento busca os schemas de saida de cada uma delas
				for (int x =0;x < this.getRecordSetInput().getRecordSetsOrigin().size();x++) {					
					this.getRecordSetInput().addSchema(this.getRecordSetInput().getRecordSetsOrigin().get(x).getSchemaData());					
				}				
			}			  
		}
		
	}
	
	


	
	public AbstractMechanism(Job jobOwner, String name, String definition) {
		super(name, definition);
		
		//this.framETLInstance = framEtl;
		
		this.jobOwner = jobOwner;
		this.addMechanismIntoJob();
		
		this.recordSetInput = new RecordSetInput(FramETL.getInstanceFramETL().getEnvironment_DSA());		
		this.recordSetOutput = new RecordSetOutput(FramETL.getInstanceFramETL().getEnvironment_DSA());
		
		
		this.init();		
								
		// TODO Auto-generated constructor stub
	}
	
	public AbstractMechanism(Job jobOwner, String name, String definition, SchemaData schema_in) {
		super(name, definition);
		
		//FramETL.getInstanceFramETL() = framEtl;
		this.jobOwner = jobOwner;
		this.addMechanismIntoJob();
		
		
		this.recordSetInput = new RecordSetInput(FramETL.getInstanceFramETL().getEnvironment_DSA());		
		this.recordSetOutput = new RecordSetOutput(FramETL.getInstanceFramETL().getEnvironment_DSA());
		
		//System.out.println("CONSTRUTUOR " + this.getClass().getName() + " Tabela IN:" + schema1_in);
		
		this.recordSetInput.addSchema(schema_in);	
		
		this.init();
								
		// TODO Auto-generated constructor stub
	}
	
	
	public AbstractMechanism(Job jobOwner, String name, String definition, String schemaSource) {
		super(name, definition);
		
	//	this.framETLInstance = framEtl;
		this.jobOwner = jobOwner;
		this.addMechanismIntoJob();
		
		
		this.recordSetInput = new RecordSetInput(FramETL.getInstanceFramETL().getEnvironment_DSA());		
		this.recordSetOutput = new RecordSetOutput(FramETL.getInstanceFramETL().getEnvironment_DSA());
		
		//System.out.println("CONSTRUTUOR " + this.getClass().getName() + " Tabela IN:" + schema1_in);
		
		SchemaData schemaIn = jobOwner.getSchemaFromEnvSource(schemaSource);
		
		this.recordSetInput.addSchema(schemaIn);	
		
		this.init();
								
		// TODO Auto-generated constructor stub
	}
	

	
	public AbstractMechanism(Job jobOwner, String name, String definition, SchemaData schema_in, SchemaData schema_out) {
		super(name, definition);
		
	//	this.framETLInstance = framEtl;
		this.jobOwner = jobOwner;
		this.addMechanismIntoJob();
		
		
		this.recordSetInput = new RecordSetInput(FramETL.getInstanceFramETL().getEnvironment_DSA());		
		this.recordSetOutput = new RecordSetOutput(FramETL.getInstanceFramETL().getEnvironment_DSA());
		
		//System.out.println("CONSTRUTUOR " + this.getClass().getName() + " Tabela IN:" + schema1_in);
		
		this.recordSetInput.addSchema(schema_in);					
		this.recordSetOutput.setSchemaData(schema_out);
		
		this.init();

	}
	
	

	
		
	public AbstractMechanism(Job jobOwner, String name, String definition, AbstractMechanism transfIn) {
		super(name, definition);
		
		
		//this.framETLInstance = framEtl;
		this.jobOwner = jobOwner;
		this.addMechanismIntoJob();
		
		
		this.recordSetInput = new RecordSetInput(FramETL.getInstanceFramETL().getEnvironment_DSA());		
		this.recordSetOutput = new RecordSetOutput(FramETL.getInstanceFramETL().getEnvironment_DSA());
				
		this.addTransfInput(transfIn);		
		
		this.init();
								
		// TODO Auto-generated constructor stub
	}
	
	public AbstractMechanism(Job jobOwner, AbstractMechanism transfIn, SchemaData schema_out) {
		
		super("", "");
		
		
		
		//this.framETLInstance = framEtl;		
		this.jobOwner = jobOwner;
		this.addMechanismIntoJob();
		
		
		this.recordSetInput = new RecordSetInput(FramETL.getInstanceFramETL().getEnvironment_DSA());		
		this.recordSetOutput = new RecordSetOutput(FramETL.getInstanceFramETL().getEnvironment_DSA());
		
		this.addTransfInput(transfIn);
		
		this.recordSetOutput.setSchemaData(schema_out);
		
		this.init();
								
		// TODO Auto-generated constructor stub
	}
	
	
	public AbstractMechanism(Job jobOwner, AbstractMechanism transfIn, String schemaTarget) {
		
		super("", "");
		
		
		
		//this.framETLInstance = framEtl;		
		this.jobOwner = jobOwner;
		this.addMechanismIntoJob();
		
		
		this.recordSetInput = new RecordSetInput(FramETL.getInstanceFramETL().getEnvironment_DSA());		
		this.recordSetOutput = new RecordSetOutput(FramETL.getInstanceFramETL().getEnvironment_DSA());
		
		this.addTransfInput(transfIn);
		
		
		SchemaData schemaOut = jobOwner.getSchemaFromEnvTarget(schemaTarget);
		
		this.recordSetOutput.setSchemaData(schemaOut);
		
		this.init();
								
		// TODO Auto-generated constructor stub
	}
	
	
	
	

	final public boolean execute(AbstractMechanism last_mechanism) {
						
		System.out.println("");
		System.out.println("----------------------------------------------------------------------");
		System.out.println(">>> Performing Transformation: " +  this.getName());

		if (last_mechanism != null) 
		   this.identifyInputSchemas(last_mechanism);
		else
		   this.identifyInputSchemas(null);
		
		this.identifyInputColumns();

		this.onOpenDataSet();

		this.prepareSchemaOutput();
		
			
		
		nAcceptedRecords = 0;
		//		555
		this.dispare_beforePerformTransformation(this);
		
	
		
		this.getRecordSetInput().refreshDataCache();
		
		if (this.recordSetInput.getRecordSet() == null) {
			return false;
		}			
		
	
		this.defineMappersColumnsType();
	

					
		this.objOutput = this.prepareOutputStructure();
		
		
		System.out.println("mappers: " + this.getMappers().size());

		
				
		// LOOPING nos registros do DataSet  
		boolean lHasRecord = this.recordSetInput.firstRow(false);
						
		if (lHasRecord)
		do {
			
			
			this.fillMappersColumnsValue();
			
			
			RowControl myRowControl =  new RowControl(); 
							
			this.dispare_forEachInputtRow(this.mappers, myRowControl);
										
			if (myRowControl.status == tp_status.REJECTED) {
				lHasRecord = this.recordSetInput.nextRow(false);
				continue;
			}
							
						
			this.storeOutPutRow();
			

							
			if (myRowControl.flow_sequence == tp_flow_sequence.FORWARD) {
				lHasRecord = this.recordSetInput.nextRow(false);		
			} 
			else if (myRowControl.flow_sequence == tp_flow_sequence.NOTHING) {				
		
				continue;
			}				
			else if (myRowControl.flow_sequence == tp_flow_sequence.BACK) {
				lHasRecord = this.recordSetInput.previostRow(false);				
			}
					
		} while (lHasRecord);
				
		
		
		System.out.println(Integer.toString(nAcceptedRecords).toString() + " Transformeds Records");
					
		this.dispare_afterPerformTransformation(this);
		

		return false;
	}
	
	
	
	
	final public SchemaData prepareSchemaOutput() { 
				
		// DATA OUTPUT
		
		
		if (this.getSchemaOutput() == null)
			this.getRecordSetOutput().setSchemaData(new SchemaData("stg_" + this.getName(),"Stage Table"));	
		
		this.recordSetOutput.prepareCursor(this.getRecordSetOutput().getSchemaData().getName(), this.getMappers());
		
		///this.recordSetOutput.prepareCursor(this.getRecordSetOutput().getSchema().getName(), this.recordSetInput.getSqlInstrution().getFieldsAs());
						
		return this.getRecordSetOutput().getSchemaData();
						
		//this.getRecordSetOutput() .refreshCursor(sqlInsert);
		
		
		//sqlInsert.defineStructureDDL((SQLInstructionSelect)this.recordSetInput.getSqlInstrution());
		
		////////////////////////////////////////////////////////////		
	}
	
	final public PreparedStatement prepareOutputStructure() {
		
		this.recordSetOutput.defineColumns(this.recordSetInput.getMetadata(),this.recordSetOutput.getSchemaData(),this.mappers);
				
		// inclui campos calculados como colunas do ResultSetOutput
		for (int x = 0; x < mappers.size(); x++) {
			
	    	if (mappers.get(x).getColumn_in() == null) {
			//	System.out.println("No considerou Mapper com Coluna Input NULL em prepareOutputStructure");
				continue;
			}
			
			if (mappers.get(x).getColumn_in().getColumn_type() == tp_column_type.VIRTUAL) {				
				this.recordSetOutput.getColumns().add(mappers.get(x).getColumn_out());
			}
			else if (mappers.get(x).getColumn_out() != null) {
				if (mappers.get(x).getColumn_out().getColumn_type() == tp_column_type.VIRTUAL)
				   this.recordSetOutput.getColumns().add(mappers.get(x).getColumn_out());
			}			
		}
		
		
		System.out.println("COLUNAS OUTPUT:::::::" + this.getRecordSetOutput().getColumns().size());
				
		// define schema repository structure
		if (this.recordSetOutput.getEnvironment().getRole() == Environment.tp_environment_role.STAGE)  {
			
			System.out.println(" STAGE MONTA CREATE");
			
			this.getRecordSetOutput().createDataObject(this.recordSetInput,this.mappers);
			
			
			
		}
			
					
		return this.getRecordSetOutput().getDataCursor(this.mappers);		
	}

	
	
	
	final public boolean storeOutPutRow() {
		
		
		nAcceptedRecords++;
			
			// looping nos maperadores (Colunas)

		/*    	
	    if(this.objOutput.getMetaData() == null)
		       System.out.println("METADATA NULOOOOOOOOOOOOOO");
		       */
	    
		//for (int i = 0; i < this.objOutput.getMetaData().getColumnCount(); i++) {
		
			
	    for (int i = 0; i < this.getRecordSetOutput().getColumns().size(); i++) {
	    	
	    //	System.out.println("varrendo: " + this.getRecordSetOutput().getColumns().get(i).getName());
	    	
			Mapper map = this.getMappers().findMapperByColOut(this.getRecordSetOutput().getColumns().get(i).getName());
			
			this.dispare_onMapping(map);
			
			
		
			if (map.getColumn_out() != null)
				this.putValue(i+1, map.getColumn_out().getData_value(), map.getColumn_out().getData_type());
			else
			 	System.out.println("COLUNA OUT ESTA NULA");
			
			   

			
			/*
		    cSourceValue = this.getRecordSetInput().findColunm(colName).getData_value();
					    
			if (map.getColumn_in().getColumn_type() == tp_column_type.CALCULETED)
				cSourceValue = map.getColumn_in().getDefaultValue();
				
			String cValue = this.onMapping(map);
			
			if (cValue == null) 
			    cValue = cSourceValue;				

			this.putValue(i+1, cValue, map.getColumn_in().getData_type());
			*/
				
			/*
			if (map.getColumn_in().getColumn_type() == tp_column_type.DATA)
				    this.putValue(i+1, cValue, data.getMetaData());
			else
				this.putValue(i+1, cValue, map.getColumn_in().getData_type());
			*/
							
		}

	    
	    try {
			
			this.getCursorOutput().execute();
			
		} catch (SQLException e) {
			// TODO Auto-generated catch block 
			//e.printStackTrace();
			System.out.println("Erro: " + e.toString());
		}
		
		
		return true;		    
			
			
	}
	
	
	private void putValue(int indexColumn, String cValue, tp_data_type columnDataType) {
				
		try {
			
			
			if (columnDataType == tp_data_type.STRING) {
				this.objOutput.setString(indexColumn,cValue);
			}
			
			else if (columnDataType == tp_data_type.BOOLEAN) {
				this.objOutput.setString(indexColumn,cValue);
			}
									
			else if (columnDataType == tp_data_type.INTEGER) {
								
				if ((cValue == null) || cValue.equalsIgnoreCase("")) 
					cValue = "0";	
				
				//System.out.println("Integer: " + cValue);
				this.objOutput.setInt(indexColumn,Integer.parseInt(cValue));
			}
			
			else if (columnDataType == tp_data_type.SURROGATE_KEY) {
				
				if ((cValue == null) || cValue.equalsIgnoreCase("")) 
					cValue = "0";	
				
				//System.out.println("Integer: " + cValue);
				this.objOutput.setInt(indexColumn,Integer.parseInt(cValue));
			}
			
										 
			else if (columnDataType == tp_data_type.DOUBLE) { 
				
				if ((cValue == null) || cValue.equalsIgnoreCase(""))
					cValue = "0";	
					
				//System.out.println("Double: " + cValue);
				this.objOutput.setDouble(indexColumn,Double.parseDouble(cValue));
			}
			
			else if (columnDataType == tp_data_type.FLOAT) { 
				
				if ((cValue == null) || cValue.equalsIgnoreCase(""))
					cValue = "0";	
				
				//System.out.println("Float: " + cValue);											    					 
				this.objOutput.setFloat(indexColumn,Float.parseFloat(cValue));
			}
			else if (columnDataType == tp_data_type.LONG) { 
				
				if ((cValue == null) || cValue.equalsIgnoreCase(""))
					cValue = "0";	
				
				//System.out.println("Long: " + cValue);											    					 
				this.objOutput.setFloat(indexColumn,Long.parseLong(cValue));
			}
			
			
			
			else if (columnDataType == tp_data_type.DATE) { 
				
				/*
				SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
				String date = dateFormatter.format(cValue);
				System.out.println("aaaaaaaaaaa");
				this.objOutput.setString(indexColumn,date);
				*/
				
				this.objOutput.setString(indexColumn, cValue);
				
			}
			
			else {
				System.out.println("TIPO NO RECONHECIDO: " + columnDataType.toString() + " Valor: " + cValue);	
			}
			
			
		} catch (SQLException e) {
			System.out.println("Erro: " + e.toString());
		}
		
	}
	
	private void putValue(int indexColumn, String cValue, ResultSetMetaData metadata) {
		
		try {
			//System.out.println("mapper: " + Integer.toString(i) + " " + map.getColumn_in().getName());				
		
			//int x = i+1;

			//|| (map.getColumn_in().getColumn_type()
								
			
			if (metadata.getColumnType(indexColumn) == 1 || metadata.getColumnType(indexColumn) == 12) {
				
				//this.objOutput.setString(x,data.getString(colName));
				this.objOutput.setString(indexColumn,cValue);
			}
									
			else if (metadata.getColumnType(indexColumn) == 4 || metadata.getColumnType(indexColumn) == 5) {
				
				
				if ((cValue == null) || cValue.equalsIgnoreCase("")) 
					cValue = "0";	
				
				//this.objOutput.setInt(x,data.getInt(colName));	
				this.objOutput.setInt(indexColumn,Integer.parseInt(cValue));
			}
										 
			else if (metadata.getColumnType(indexColumn) == 2 || metadata.getColumnType(indexColumn) == 3 || metadata.getColumnType(indexColumn) == 8) { 
				
				if ((cValue == null) || cValue.equalsIgnoreCase(""))
						cValue = "0";	
					
				//this.objOutput.setDouble(x,data.getDouble(colName));						    					 
				this.objOutput.setDouble(indexColumn,Double.parseDouble(cValue));
			}
			
			else if (metadata.getColumnType(indexColumn) == 6) { 
				 
				if ((cValue == null) || cValue.equalsIgnoreCase("")) 
						cValue = "0";	
					
				//this.objOutput.setFloat(x,data.getFloat(colName));						    					 
				this.objOutput.setFloat(indexColumn,Float.parseFloat(cValue));
			}

			else if (metadata.getColumnType(indexColumn) == 7) { 
				 
				if ((cValue == null) || cValue.equalsIgnoreCase(""))  
						cValue = "0";	
					
				//this.objOutput.setLong(x,data.getLong(colName));						    					 
				this.objOutput.setLong(indexColumn,Long.parseLong(cValue));
			}
			
			else if ((metadata.getColumnType(indexColumn) == 91) ||  (metadata.getColumnType(indexColumn) == 93)) { 
				 
				if ((cValue == null) || cValue.equalsIgnoreCase(""))  
					cValue = "0";
				
				//cValue = "25/01/11";
				
				/*
				System.out.println("TIPO DATAAAAAAAAAAAAAAAAAAA: " +  cValue);				
				SimpleDateFormat dateFormatter = new SimpleDateFormat("dd/MM/yyyy");
				String date = dateFormatter.format(cValue);
				*/
				
							
				//this.objOutput.setDate(indexColumn,date);
				this.objOutput.setString(indexColumn, cValue);
				
			}
		
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			//e.printStackTrace();
			System.out.println("Erro: " + e.toString());
		}
	
		
		
	}
	
	
	




	


	
	
	public ResultSet getDataInput() {
		
		return this.getRecordSetInput().getRecordSet();
	}
	
	public PreparedStatement getCursorOutput() {
	
		return this.getRecordSetOutput().getDataCursor(this.mappers);
		
	}
	
	
	final public void setOutputVal(int index, int val) {
		
		try {
			this.getRecordSetInput().getDataCursor(this.mappers).setInt(index, val);
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	final public void setOutputVal(int index, Double val) {
	
	try {
		this.getRecordSetInput().getDataCursor(this.mappers).setDouble(index, val);
	} catch (SQLException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	
	}


	
	
	// MAPPERS
	
	public ListOfMapper getMappers() {
		return mappers;
	}


	public void setMappers(ListOfMapper mappers) {
		this.mappers = mappers;
	}
		

	//////

	
	
	// PARAMS
	public ListOfParams getParams() {
		return params;
	}
	
	public void setParams(ListOfParams params) {
		this.params = params;
	}
		

	
	public Params getParam(int index) {
		return this.params.get(index);
	}
	
	
	
	
	/* veer
	public void addMapper(Column column_in, Column column_out, String operator) {
		
		DataInstanceColumn myDataInstance = new DataInstanceColumn(column_in.getName(),column_in.getType(),"",this.getRecordSetInput());
		
		this.getMappers().add(myDataInstance, column_out, operator);		
	}
	*/
	



	

	
	


	public Mapper addMapper(Attribute column_in) {
				
		//Column myDataInstance = new Column(column_in.getName(),column_in.getType(),"",this.getRecordSetInput());
		
		// se ja tiver no inclui de novo
	//	if (!allowRepeateddOut)
		Mapper map = this.getMappers().findMapperByColOut(column_in.getName());
		
		if (map != null)
		   return map;
		
		
		//Column column_out = null;		
		//if (column_in.getColumn_type() == tp_column_type.DATA)
		
		Attribute column_out = new Attribute(column_in.getName(),this.getRecordSetOutput(),column_in.getData_type());
			
			
		this.getMappers().add(column_in, column_out);
		
		map = this.getMappers().get(this.getMappers().size()-1);
		
		return map;
	}
	
	


	

	// FIM PARAMS
	
	public RecordSetInput getRecordSetInput() {
		return recordSetInput;
	}


	public void setRecordSetInput(RecordSetInput recordSetInput) {
		this.recordSetInput = recordSetInput;
	}
	
	public void setRecordSetInput(Environment environment) {
		this.recordSetInput = new RecordSetInput(environment);
	}


	public RecordSetOutput getRecordSetOutput() {
		return recordSetOutput;
	}
	
	public void setRecordSetOutput(Environment environment) {
		this.recordSetOutput = new RecordSetOutput(environment);
	}


	public void setRecordSetOutput(RecordSetOutput recordSetOutput) {
		this.recordSetOutput = recordSetOutput;
	}
	

	
	public SchemaData getSchemaOutput() {
	
		return this.getRecordSetOutput().getSchemaData();
	}
	
	public SchemaData getSchemaInput(int schemaIndex) {
		
		return this.getRecordSetInput().getSchema(schemaIndex);
	}
	
	public List<SchemaData> getSchemasInput() {
		
		return this.getRecordSetInput().getSchemasData();
	}
	
	
	final protected AbstractMechanism getTransfInput(int transfIndex) {
		return transfInput.get(transfIndex);
	}
	
	
	protected void addTransfInput(AbstractMechanism transf) {
		
		this.transfInput.add(transf);
		this.recordSetInput.addRecordSetOrigin(transf.getRecordSetOutput());
	}
	

	
	final public void setTransfInputSecondary(AbstractMechanism transfInputSecondary) {
		
		this.addTransfInput(transfInputSecondary);
		
		this.getRecordSetInput().addRecordSetOrigin(transfInputSecondary.getRecordSetOutput());
		
		//System.out.println(this.getName() + " setando tabela 2:::" + transfInputSecondary.getRecordSetOutput().toString());
	}
	
	
	public void addInputColumnsFromInventaryOrTransf() {
	
		    List<Attribute> columns = null;
		    		    
		    // para o caso das transformaes que so consideram os campos do primeiro input no select
		    	    
		    if (this.getTransfInput().size() > 0) {		    	
				columns = this.transfInput.get(0).getRecordSetOutput().getColumns();
				
				
				
			}				    
			else if (this.getRecordSetInput().getSchema(0) != null) {
				System.out.println("Criou colunas a partir do inventario");
				columns = this.getRecordSetInput().getSchema(0).getColumns_Inventory();					
			}
		    	
			this.getRecordSetInput().setColumns(columns);
										
	}
	
	public void addInputColumnsFromMappers() {
		
	    List<Attribute> cols = new ArrayList<Attribute>();
	    	    
	    for (int x = 0; x < mappers.size(); x++) {
			
	    	if (mappers.get(x).getColumn_in() == null) {
		//		System.out.println("No considerou Mapper com Coluna Input NULL");
				continue;
			}
	    	
	    	/*
			if (mappers.get(x).getColumn_in().getColumn_type() == tp_column_type.CALCULETED) {
				System.out.println("mapper colin calculeted nao criou coluna: " + mappers.get(x).getColumn_in().getName());
				continue;
			}
			*/
	    	
	    	cols.add(mappers.get(x).getColumn_in());
	    }			

	    this.getRecordSetInput().setColumns(cols);
									
	}
	
	
	
	final public void addMappersFromInputColumns() {
			
	    List<Attribute> columns = this.getRecordSetInput().getColumns();
			
		if (columns != null)
		for (int x = 0; x < columns.size();x++) {					

			// mudana mario
			
			// tenta usar o schamadata informado na coluna, se no houver usa o schema do input padro
			SchemaData schemadata = columns.get(x).getSchema();			
			if ((schemadata == null) || (schemadata.getName().equalsIgnoreCase(""))) 			
				schemadata = this.getRecordSetInput().getSchemasData().get(0);
					
			// mario domingo de carnaval
			// verifica se o mapper ja foi incluido
			if (this.getMappers().findMapperByColIn(columns.get(x).getName()) == null) {
				this.addMapper(new Attribute(columns.get(x).getName(),schemadata,columns.get(x).getData_type()));
				   System.out.println("incluiu no mapper: " + columns.get(x).getName());
			}
			   
		}
	}
	
	
	final public void defineMappersColumnsType() {
	   	
		for (int x = 0; x < this.recordSetInput.getColumns().size();x++) {
    		
    		Attribute col = this.recordSetInput.getColumns().get(x);	
    		
    		// mario na segunda de carnaval
    		//Mapper map = this.getMappers().findMapperByColOut(col.getName());
    		Mapper map = this.getMappers().findMapperByColIn(col.getName());
    		    		
    		
    		if (map != null) {
    			map.getColumn_in().setData_type(col.getData_type());
    			    			
    			// torna tipos o input e output iguais
    			if (map.getColumn_out() != null) {   // para os casos de campos de CONCAT  e outros no h coluna out

        			if (map.getColumn_out().getColumn_type()  ==  tp_column_type.DATA)
        			    map.getColumn_out().setData_type(col.getData_type());
        				
    			}
    			
    			
                // System.out.println("Mapeador:  " +map.getColumn_in().getName() + " " + map.getColumn_in().getData_type());
    		}
               
    	}
	 	
	}
	
	public void fillMappersColumnsValue() {
		
		ResultSet data = this.getRecordSetInput().getRecordSet();

		for (int i = 0; i < this.mappers.size();i++) {
		
			Attribute colInput = this.mappers.get(i).getColumn_in();
			Attribute colOutput = this.mappers.get(i).getColumn_out();
			
			
			if (colOutput != null) 
				colOutput.setData_value(colOutput.getData_value());
						
			
			if (colInput == null) 
				continue;
			
			colInput.setData_value("");
			
						
			// se for campo calculado vai dar erro
			try{					
			    
				if (colInput.getColumn_type() == tp_column_type.DATA) {	
					
					// em 04/03
					//String cColAs = colInput.getName();
					// foi necessrio pois no caso do CSV o columnnLAbe vem no formato columnLabel tem ao format [tabela.campo]
					
					String cColAs = colInput.getName(); 
										
					if (colOutput != null) {
						cColAs = colOutput.getName(); //.getInternalName();
					}
					
		
					//System.out.println("campo: " + colOutput.getName());
					//System.out.println("campo: " + colOutput.getInternalName());
					
					/*System.out.println("valor do banco1: " + data.getString(1));
					System.out.println("valor do banco2: " + data.getString(2));
					System.out.println("valor do banco3: " + data.getString(3));
					*/
					
					
					colInput.setData_value(data.getString(cColAs)); // o as na consulta j traz o campo INPUT com o novo nome = ao campo output
				}			        			    
				else if (colInput.getColumn_type() == tp_column_type.VIRTUAL) {
					
					 colInput.setData_value(colInput.getDefaultValue());
					 //System.out.println("setou valor default do camp calculado: " + colInput.getName() + " = " + colInput.getData_value());
						
				}
					
				
				if (colOutput != null)  {				   
					colOutput.setData_value(colInput.getData_value());				
				}
								
			    
			} catch (SQLException e) {
				System.out.println("Erro ao tentar consultar dataset input: " + e.toString());	
			}														
		}
	}

	
	/*  Alternativa ao uso de transformaes baseadas em SQL para carrego em memoria
	public void fillRowsValue(List<String> rowIn, List<String> rowOut) {
		
		ResultSet data = this.getRecordSetInput().getDataCache();

		for (int i = 0; i < this.mappers.size();i++) {
		
			Column colInput = this.mappers.get(i).getColumn_in();
			Column colOutput = this.mappers.get(i).getColumn_out();
			
			rowIn.add("");
			rowOut.add("");
			
			
			if (colOutput != null)  {
				
				colOutput.setData_value(colOutput.getData_value());						
			    rowOut.set(i, colOutput.getData_value());
			}
						
			
			if (colInput == null) {					
				continue;
			}
				
			
			colInput.setData_value("");
			
		    rowIn.set(i, colInput.getData_value());
						
			// se for campo calculado vai dar erro
			try{					
			    
				if (colInput.getColumn_type() == tp_column_type.DATA) {	
					
					// em 04/03
					//String cColAs = colInput.getName();
					// foi necessrio pois no caso do CSV o columnnLAbe vem no formato columnLabel tem ao format [tabela.campo]
					
					String cColAs = colInput.getName(); 
										
					if (colOutput != null) {
						cColAs = colOutput.getName(); //.getInternalName();
					}
					
					
					colInput.setData_value(data.getString(cColAs)); // o as na consulta j traz o campo INPUT com o novo nome = ao campo output
				}			        			    
				else if (colInput.getColumn_type() == tp_column_type.CALCULETED) {
					
					 colInput.setData_value(colInput.getDefaultValue());
					 //System.out.println("setou valor default do camp calculado: " + colInput.getName() + " = " + colInput.getData_value());
						
				}
					
				rowIn.set(i, colInput.getData_value());
				
				if (colOutput != null)  {				   
					colOutput.setData_value(colInput.getData_value());
					
					rowOut.set(i, colInput.getData_value());
				}
								
			    
			} catch (SQLException e) {
				System.out.println("AQUI Erro ao tentar consultar dataset input: " + e.toString());	
			}														
		}		
		
	}
	

	
	public void fillMapperValue(List<String> rowIn, List<String> rowOut) {

		for (int i = 0; i < this.mappers.size();i++) {
		
			Column colInput = this.mappers.get(i).getColumn_in();
			Column colOutput = this.mappers.get(i).getColumn_out();
			
			if (colOutput != null)  {				
				colOutput.setData_value(rowOut.get(i));
			}
					
			if (colInput != null) {					
				colInput.setData_value(rowIn.get(i));
			}
		}
						
		
	}
	
	*/

			
	
	public List<AbstractMechanism> getTransfInput() {
		return transfInput;
	}

	
	/*
	public SQLSyntaxStandard getInputStatement() {

		return this.getRecordSetInput().getSqlGenerator();
		
	}
	*/
	
	public SQLElements getInputSqlElements() {
		
		return this.getRecordSetInput().getSqlGenerator().getSqlElements();
	}
	
	
	public List<AbstractSQLInstruction> getInputSecondaryStatements() {

		return this.getRecordSetInput().getSqlGenerator().getSqlElementsSecondary();
		
		
	}
	
	
	public Attribute getColumn_source(String columnName) {
		
		return this.getRecordSetInput().getSchema(0).findColumn(columnName);			
		
	}
	
	public Attribute columnDataByName(String columnName) {
		
		return new Attribute(columnName,this);
		
	}
	
	
	public Attribute columnVirtual(String columnName, tp_data_type dataType, String defaultValue, int size) {		
		return new Attribute(columnName, "","",dataType,tp_column_type.VIRTUAL, defaultValue,size);		
	}
	
	
	
	public Mapper createSurrogateKey() {
		
		String cSKName =  this.concatsurrogateKeyName();
		
		Attribute column_in = new Attribute(cSKName, "","",tp_data_type.SURROGATE_KEY,tp_column_type.VIRTUAL,"",11);
			
		Attribute column_out = new Attribute(cSKName, "","",tp_data_type.SURROGATE_KEY,tp_column_type.VIRTUAL,"",11);
					
		this.getMappers().add(column_in, column_out);
		
		Mapper map = this.getMappers().get(this.getMappers().size()-1);
		
		return map;
		
	}
	
	public String concatsurrogateKeyName() {
		
		return this.getName() + "_SK";
	}
	
	
	public Attribute columnTarget(String columnName) {
		
		return new Attribute(columnName,this);
		
	}

	

	public final void dispare_afterPerformTransformation(AbstractMechanism transf) {
		// TODO Auto-generated method stub
		
		this.onFinish(transf);
		
		for (int x =0;x < this.customHandle.size();x++) {					
			this.customHandle.get(x).onFinish(transf);					
		}	
		
	}


	public final  void dispare_beforePerformTransformation(AbstractMechanism transf) {
		// TODO Auto-generated method stub
		
		this.onStart(transf);
		
		for (int x =0;x < this.customHandle.size();x++) {					
			this.customHandle.get(x).onStart(transf);					
		}	
				
	}


	public void  dispare_forEachInputtRow(ListOfMapper mappers, RowControl rowControl) {	
		
		this.forEachInputtRow(mappers, rowControl);
		
		for (int x =0;x < this.customHandle.size();x++) {					
			this.customHandle.get(x).forEachInputtRow(mappers, rowControl);					
		}
	
	}


	public final void dispare_onMapping(Mapper map) {
		
		this.onMapping(map);
		
		for (int x =0;x < this.customHandle.size();x++) {					
			this.customHandle.get(x).onMapping(map);					
		}
	}
	
	
	public void loadAddAllAttributesFromInput(boolean useAllColumnsFromFlow) {
		this.l_loadAllInputAttributes = useAllColumnsFromFlow;
	}

	protected Job getInternalJob() {
		
		if (this.internalJob == null) 
			this.createInternalJob();
			
		return internalJob;
	}


	
	/*
	
	public void addSubTransformation(AbstractMechanism transform) {
		
		if (this.subJob == null) 
			this.createSubJob();
				
		if (this.subJob == null) {
			this.subJob = new Job("subjob","");
			this.subJob.setEnvironment_Source(this.framETLInstance.getEnvironmentSource(0));
			this.subJob.setEnvironment_Target(this.framETLInstance.getEnvironmentTarget(0));			
		}
		
		subJob.addTransformation(transform);			
	}
	*/

	public void performInternalMechanisms() {
		
		if (this.internalJob != null) 
		   internalJob.perform();
	}
	
	
	public void addMechanismIntoJob() {
		
		this.setName(this.jobOwner.getName() + "_" + this.getClass().getSimpleName() + "_" + this.jobOwner.getTransformsCount());
		this.jobOwner.addMechanism(this);
	}

	public String getTag() {
		return tag;
	}

	public void setTag(String tag) {
		this.tag = tag;
	}
	
	
	public void createInternalJob() {
		
		
		this.internalJob = new Job("subjob","");
		this.internalJob.setEnvironment_Source(FramETL.getInstanceFramETL().getEnvironmentSource(0));
		this.internalJob.setEnvironment_Target(FramETL.getInstanceFramETL().getEnvironmentTarget(0));			
		
		
		
	}
	
	

		
	
	/*
	public void setSubJob(Job subJob) {
	Job 
	*/
	


}
