package processETL_IntegrationCosts;


import frametl4CostAccount.*;




import metadata_inventory.Environment;
import metadata_inventory.SchemaData;
import metadata_inventory.Attribute.tp_data_type;
import metadata_inventory_repository_common.RepositoryFirebird;
import metadata_inventory_repository_common.RepositoryMySql;
import metadata_inventory_repository_common.RepositoryOracle;
import metadata_model_general.Batch;
import metadata_model_general.Job;
import metadata_model_mechanisms_common.Extractor;
import metadata_model_mechanisms_common.Filter;
import metadata_model_mechanisms_common.Join;
import metadata_model_mechanisms_common.Loader;
import metadata_model_mechanisms_common.Lookup;



public class IntegrationCosts extends FramETLCostAccount {
		
	
	public IntegrationCosts(boolean runProcessOnStarts) {
		super(runProcessOnStarts);
		// TODO Auto-generated constructor stub
	}

	public static void main(String[] args) {
		// TODO Auto-generated method stub
			
		System.out.println("App Main");			
		new IntegrationCosts(true);				

	}

	
	@Override
	public void defineMetadataInventory() {

		
		System.out.println("Defining Metadata Inventory....");
		
		
		// Instanciando Repositrios
		RepositoryMySql repositoryMysql = new RepositoryMySql("localhost","frametl_dsa","root", "123");				
		RepositoryOracle repositoryOracle = new RepositoryOracle("localhost", "XE", "mario", "123456");	
		RepositoryFirebird repositoryFirebird = new RepositoryFirebird("127.0.0.1", "c:/sishrimp/atlantis.fdb", "sysdba", "masterkey");
		
	
		// Instanciando Ambientes
		this.setInventoryEnv_DSA("EnvDSA","Area de processamento de Dados",repositoryMysql);	
				
		Environment envSourceFB     = this.addInventoryEnv_Source("EnvSourceFB","Uma Fonte de Dados",repositoryFirebird);
		Environment envSourceOra    = this.addInventoryEnv_Source("EnvSourceOra","Uma Fonte de Dados",repositoryOracle);
		
		Environment envTargetFB     = this.addInventoryEnv_Target("EnvTargFB"," Destino dos dados",repositoryFirebird);
				
				
		
		
		// Instanciando Tabelas (Esquemas de Dados de Origem)
		SchemaData schData = null;
		schData = envSourceFB.addSchemaData(new SchemaData("prw0100","Viveiro","Cadastro de Viveiros"));
		schData.addColumn_Inventory("codviv");
		schData.addColumn_Inventory("nome");		
		schData.addColumn_Inventory("codcul");
		schData.addColumn_Inventory("area");
		schData.addColumn_Inventory("ciclo");
		schData.addColumn_Inventory("tipo");
		
		schData = envSourceFB.addSchemaData(new SchemaData("PRW0000","Cultivo","Cadastro de Cultivos"));
		schData.addColumn_Inventory("codcul");
		schData.addColumn_Inventory("codviv");
		schData.addColumn_Inventory("ciclo");

		
		schData = envSourceFB.addSchemaData(new SchemaData("insumos_movimento","Consumo","Movimento de Insumos"));		
		schData.addColumn_Inventory("ID_INSUMO_MOVIMENTO");
		schData.addColumn_Inventory("DATA");
		schData.addColumn_Inventory("QTDTOTAL");
		schData.addColumn_Inventory("CUSTOUN");
		schData.addColumn_Inventory("CUSTOTOTAL");
		schData.addColumn_Inventory("CODIGO_INSUMO_REL");
		schData.addColumn_Inventory("PLANO_CONTAS_ID_REL");
		schData.addColumn_Inventory("CODCUL_REL");
				
		schData = envSourceFB.addSchemaData(new SchemaData("PRW1300","Despescas","Movimento de Despescas"));
		schData.addColumn_Inventory("id");
		schData.addColumn_Inventory("codcul");
		schData.addColumn_Inventory("peso");
		schData.addColumn_Inventory("data");

		schData = envSourceFB.addSchemaData(new SchemaData("PLANO_CONTAS","PlanoContas","Plano de contas do OLTP"));
		schData.addColumn_Inventory("PLANO_CONTAS_ID");
		schData.addColumn_Inventory("CONTA_RAZAO_SAP");
						
		schData = envSourceOra.addSchemaData(new SchemaData("FCAXSAP_CUSTOS_INDIRETOS","CustosIndiretos","Registro de Custos Indiretos"));
		schData.addColumn_Inventory("NUM_DOC");
		schData.addColumn_Inventory("DATA_DOC");
		schData.addColumn_Inventory("CONTA_RAZAO");
		schData.addColumn_Inventory("VALOR");
		
				
		
		// Tabelas de destino
		envTargetFB.addSchemaData(new SchemaData("insumos_movimento","CustosRateados","Lanamento de Custos"));
		
		
	}
	
		

	@Override
	public void defineMetadataProcess() {
		// TODO Auto-generated method stub
				
		System.out.println("Defining Metadata ETL....");
		
		
		// Instancia um Lote de Processos
        Batch mybatch = this.addBatch("btc_1","Batch one");

        
		// Tarefa com o ambiente Oracle como Fonte de Dados
		Job myjobFromOracle = mybatch.addJob("JOB_2","ETL Custos",this.getEnvironmentSource(1),this.getEnvironmentTarget(0));
		
       		
		// Tarefa com o ambiente Firebird como Fonte de Dados
		Job myjobFromFirebird = mybatch.addJob("JOB_1","ETL Custos",this.getEnvironmentSource(0),this.getEnvironmentTarget(0));
						
				
		
		// Define as Transformaes (Mecanismos)
		
		// Extratores
		Extractor transfECIndiretos = new Extractor(myjobFromOracle,myjobFromOracle.getSchemaFromEnvSource("CustosIndiretos"));	
		
		Extractor transfEViveiro = new Extractor(myjobFromFirebird,myjobFromFirebird.getSchemaFromEnvSource("Viveiro"));
    	Extractor transfCultivo = new Extractor(myjobFromFirebird,myjobFromFirebird.getSchemaFromEnvSource("Cultivo")); 
    	Extractor transfEPlanoContas = new Extractor(myjobFromFirebird,myjobFromFirebird.getSchemaFromEnvSource("PlanoContas"));	
		
    	
    	// Filtra Viveiros do Tipo Engorda (Direcionadores)
        Filter transfViveirosEngorda = new Filter(myjobFromFirebird, transfEViveiro); 		
        transfViveirosEngorda.addParam("TIPO",Filter.OPERATION_IGUAL,"E");
        transfViveirosEngorda.addParam("CODCUL",Filter.OPERATION_DIFERENT,"");
    	
        
        // Join Viveiros e Cultivos (Comsumidores)
		Join transfVivCultivos = new Join(myjobFromFirebird,transfViveirosEngorda,transfCultivo); 
		transfVivCultivos.setParam("codcul","codcul");
		
		
        // Lookup Plano de Contas
        Lookup transflkpContaCIndiretos = new Lookup(myjobFromFirebird,transfECIndiretos, transfEPlanoContas); 
        transflkpContaCIndiretos.setParam("CONTA_RAZAO","CONTA_RAZAO_SAP","PLANO_CONTAS_ID");		
        
        
        
        
        // Rateio  (Mecanismo CostProration)
        CostProration transfRateio = new CostProration(myjobFromFirebird,transflkpContaCIndiretos);
		transfRateio.setParamCostConsumer(transfVivCultivos,"VALOR");
		transfRateio.setParamCostDriver(transfViveirosEngorda,"AREA");
		
		transfRateio.addMapper(transflkpContaCIndiretos.columnVirtual("CODVIV",tp_data_type.STRING,"",4));
		transfRateio.addMapper(transflkpContaCIndiretos.columnVirtual("CODCUL",tp_data_type.STRING,"",6));
		transfRateio.addMapper(transflkpContaCIndiretos.columnVirtual("CICLO",tp_data_type.INTEGER,"",6));
				
		
		
		
		// Carga dos Custos Rateados Alocados
		Loader loadCA = new Loader(myjobFromFirebird,transfRateio,myjobFromOracle.getSchemaFromEnvTarget("CustosRateados"));
		loadCA.loadAddAllAttributesFromInput(false);
		loadCA.addMapper(transfRateio.columnDataByName("VALOR"),loadCA.columnTarget("CUSTOTOTAL"));		
		loadCA.addMapper(transfRateio.columnDataByName("DATA_DOC"), loadCA.columnTarget("DATA"));
		loadCA.addMapper(transfRateio.columnDataByName("PLANO_CONTAS_ID"), loadCA.columnTarget("PLANO_CONTAS_ID_REL"));			
		loadCA.addMapper(transfRateio.columnDataByName("NUM_DOC"), loadCA.columnTarget("NUMCONTROLE"));		
		loadCA.addMapper(transfRateio.columnDataByName("CODVIV"), loadCA.columnTarget("CODVIV_REL"));
		loadCA.addMapper(transfRateio.columnDataByName("CODCUL"), loadCA.columnTarget("CODCUL_REL"));
		loadCA.addMapper(transfRateio.columnDataByName("CICLO"), loadCA.columnTarget("CICLO_REL"));					
		
		// Manipulador Customizado 
		loadCA.addCustomHandler(new ScriptFormatAttrributes());
						
	}

	
	
	
}
